@@ -1,3 +1,4 @@
+from anyio import sleep
 from .functions_projects import get_list_of_all_project_infos, get_recent_project
 from ._backend_calls import _backend_GET
 from .functions import move_parameter_placeholder
@@ -32,6 +33,8 @@ def st_project_selector(title: str = None):
     else:
         st.metric(label=f"Active project ID: {st.session_state['project_id']}",
                   value=f"{st.session_state['project_name']}")
+        
+    return st.session_state.get("project_id", None)
 
 
 def st_system_generator_selector():
@@ -41,8 +44,8 @@ def st_system_generator_selector():
     Suitable to be used in conjunction with st_project_selector().
     """
     # check if the project is set and exit without doing anything if not
-    if 'project_id' not in st.session_state or st.session_state['project_id'] is None:
-        return
+    if 'project_id' not in st.session_state:
+        return None
 
     # get the list of system generators for the current project
     status_code, system_generators = _backend_GET(f"/projects/{st.session_state['project_id']}/sysgen/")
@@ -63,14 +66,17 @@ def st_system_generator_selector():
         st.error(f"Error while loading system generators: {system_generators}")
 
     # After selecting a system generator, extract and present the generated designs from system_generators with the ID system_generators_id
-    if 'sysgen_id' in st.session_state:
-        status_code, system_generator = _backend_GET(f"/projects/{st.session_state['project_id']}/sysgen/{st.session_state['sysgen_id']}/")
-        if status_code == 200:
-            st.session_state["system_generator"] = system_generator
-            st.session_state["generated_designs"] = system_generator.get("generated_designs")
-        else:
-            st.error(f"Error while loading generated designs: {system_generator}")
+    if 'sysgen_id' not in st.session_state:
+        st.stop()
+    
+    status_code, system_generator = _backend_GET(f"/projects/{st.session_state['project_id']}/sysgen/{st.session_state['sysgen_id']}/")
+    if status_code == 200:
+        st.session_state["system_generator"] = system_generator
+        st.session_state["generated_designs"] = system_generator.get("generated_designs")
+    else:
+        st.error(f"Error while loading generated designs: {system_generator}")
 
+    return st.session_state.get("sysgen_id", None)
 
 def st_generated_design_selector():
     """A function which adds a generated design selector to the top of the page.
@@ -79,11 +85,11 @@ def st_generated_design_selector():
     Suitable to be used in conjunction with st_project_selector().
     """
     # check if the project is set and exit without doing anything if not
-    if 'project_id' not in st.session_state or st.session_state['project_id'] is None:
-        return
+    if 'project_id' not in st.session_state:
+        return None
 
     # get the list of system generators for the current project
-    status_code, system_generators = _backend_GET(f"/projects/{st.session_state['project_id']}/sysgen")
+    status_code, system_generators = _backend_GET(f"/projects/{st.session_state['project_id']}/sysgen/")
 
     # if successful display a selector for the system generators based on their IDs and names
     if status_code == 200:
@@ -113,15 +119,15 @@ def st_generated_design_selector():
     # Only show design selector and warnings if a system generator has been selected
     if 'sysgen_id' in st.session_state:
         if 'generated_designs' in st.session_state and st.session_state["generated_designs"]:
-            generated_designs = st.selectbox("Generated Design",
+            selected_generated_design = st.selectbox("Generated Design",
                                              [(design["id"], design["comp_list_uids"])
                                               for design in st.session_state["generated_designs"]],
                                              index=None,
                                              placeholder="Select a generated design",
                                              format_func=lambda x: f"Design #{x[0]}, Components: {x[1]}")
-            if generated_designs is not None:
-                st.session_state["generated_design_id"] = generated_designs[0]
-                st.session_state["generated_design_comp_list_uids"] = generated_designs[1]
+            if selected_generated_design is not None:
+                st.session_state["generated_design_id"] = selected_generated_design[0]
+                st.session_state["generated_design_comp_list_uids"] = selected_generated_design[1]
         else:
             st.warning("No generated designs available for the selected system generator.")
 
@@ -134,18 +140,20 @@ def st_generated_design_selector():
     elif 'sysgen_description' in st.session_state:
         st.metric(label="Selected System Generator",
                   value=st.session_state["sysgen_description"])
+    
+    return st.session_state.get("generated_design_id", None)
 
 
-def st_load_components(uid_list: list[str] = None) -> None:
+def st_load_components(uid_list: list[str] = None) -> dict[str, dict]:
     """Loads the defined UIDs from the backend into the session state under session_state['db']['components'][{uid}]
-    If no list of UIDs is given, check the UIDs in generated_design_comp_list_uids and load them instead.
+    If no list of UIDs is given, check the current projects system generators and iterate over all generated designs with their comp_list_uids.
 
     Displays a cute spinner while loading.
     """
 
     # exit if no data is given to act upon (selection has not been made previously)
-    if 'generated_design_comp_list_uids' not in st.session_state and uid_list is None:
-        return
+    if 'project_id' not in st.session_state and uid_list is None:
+        return {}
 
     # create sessions state db if not exists
     if 'db' not in st.session_state:
@@ -155,13 +163,23 @@ def st_load_components(uid_list: list[str] = None) -> None:
 
     # show infobox with loading message
     with st.spinner("Loading components..."):
-        if uid_list is None and 'generated_design_comp_list_uids' in st.session_state:
-                for component_uid in st.session_state['generated_design_comp_list_uids']:
-                    status_code, component = _backend_GET(f"/v2/components/{component_uid}?exclude_unset=false")
-                    if status_code == 200:
-                        st.session_state['db']['components'][component_uid] = component
-                    else:
-                        st.error(f"Error loading component {component_uid}: {component}")
+        if uid_list is None and 'project_id' in st.session_state:
+            status_code, system_generators = _backend_GET(f"/projects/{st.session_state['project_id']}/sysgen/")
+            for sysgen in system_generators:
+                status_code, system_generator = _backend_GET(f"/projects/{st.session_state['project_id']}/sysgen/{sysgen['id']}/")
+                if status_code == 200:
+                    for design in system_generator.get("generated_designs", []):
+                        for component_uid in design.get("comp_list_uids", []):
+                            if component_uid not in st.session_state['db']['components']:
+                                status_code, component = _backend_GET(f"/v2/components/{component_uid}")
+                                if status_code == 200 and isinstance(component, dict):
+                                    st.session_state['db']['components'][component_uid] = component
+                                    if component_uid != component.get("uid"):
+                                        st.session_state['db']['components'][component['uid']] = component_uid
+                                else:
+                                    st.error(f"Error loading component {component_uid}: {component}")
+                else:
+                    st.error(f"Error loading system generator {sysgen['id']}: {system_generator}")
 
         else:
             # Load each component by UID
@@ -172,5 +190,13 @@ def st_load_components(uid_list: list[str] = None) -> None:
                 else:
                     st.error(f"Error loading component {component_uid}: {component}")
 
+
     for component_uid, component in st.session_state['db']['components'].items():
-        st.session_state['db']['components'][component_uid] = move_parameter_placeholder(component)
\ No newline at end of file
+        try:
+            st.session_state['db']['components'][component_uid] = move_parameter_placeholder(component)
+        except AttributeError as e:
+            st.error(f"Error processing component {component_uid}: {e}")
+            st.error(component)
+            st.error(type(component))
+
+    return st.session_state['db']['components']
\ No newline at end of file
